package com.vitorpamplona.amethyst.service

import com.vitorpamplona.amethyst.model.Account
import com.vitorpamplona.amethyst.service.model.ChannelMessageEvent
import com.vitorpamplona.amethyst.service.model.LnZapEvent
import com.vitorpamplona.amethyst.service.model.ReactionEvent
import com.vitorpamplona.amethyst.service.model.ReportEvent
import com.vitorpamplona.amethyst.service.model.RepostEvent
import com.vitorpamplona.amethyst.service.relays.FeedType
import com.vitorpamplona.amethyst.service.relays.TypedFilter
import nostr.postr.JsonFilter
import com.vitorpamplona.amethyst.service.model.ContactListEvent
import com.vitorpamplona.amethyst.service.model.MetadataEvent
import com.vitorpamplona.amethyst.service.model.TextNoteEvent

object NostrAccountDataSource: NostrDataSource("AccountData") {
  lateinit var account: Account

  fun createAccountContactListFilter(): TypedFilter {
    return TypedFilter(
      types = FeedType.values().toSet(),
      filter = JsonFilter(
        kinds = listOf(ContactListEvent.kind),
        authors = listOf(account.userProfile().pubkeyHex),
        limit = 1
      )
    )
  }

  fun createAccountMetadataFilter(): TypedFilter {
    return TypedFilter(
      types = FeedType.values().toSet(),
      filter = JsonFilter(
        kinds = listOf(MetadataEvent.kind),
        authors = listOf(account.userProfile().pubkeyHex),
        limit = 1
      )
    )
  }

  fun createAccountReportsFilter(): TypedFilter {
    return TypedFilter(
      types = FeedType.values().toSet(),
      filter = JsonFilter(
        kinds = listOf(ReportEvent.kind),
        authors = listOf(account.userProfile().pubkeyHex)
      )
    )
  }

  fun createNotificationFilter() = TypedFilter(
    types = FeedType.values().toSet(),
    filter = JsonFilter(
      kinds = listOf(
        TextNoteEvent.kind, ReactionEvent.kind, RepostEvent.kind, ReportEvent.kind, LnZapEvent.kind, ChannelMessageEvent.kind
      ),
      tags = mapOf("p" to listOf(account.userProfile().pubkeyHex)),
      limit = 200
    )
  )

  val accountChannel = requestNewChannel()

  override fun updateChannelFilters() {
    // gets everthing about the user logged in
    accountChannel.typedFilters = listOf(
      createAccountMetadataFilter(),
      createAccountContactListFilter(),
      createNotificationFilter(),
      createAccountReportsFilter()
    ).ifEmpty { null }
  }
}